package com.jigoku.backend;

import com.badlogic.gdx.InputProcessor;
import com.jigoku.stages.LevelStage;

public class Director implements InputProcessor {
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	private AbstractStage currentStage;
	private boolean isPaused;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public Director() {
		currentStage = null;
		isPaused = false;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// MUTATORS / SETTERS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void setStage(AbstractStage newStage) {
		if(newStage.director != this)
			return;
		currentStage = newStage;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// ACCESSORS / GETTERS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public boolean isPaused() {
		return isPaused;
	}
	
	public AbstractStage getCurrentStage() {
		return currentStage;
	}
	
	public LevelStage getCurrentLevel() {
		if(currentStage instanceof LevelStage)
			return (LevelStage)currentStage;
		else
			return null;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// OVERRIDES
	///////////////////////////////////////////////////////////////////////////////////////////////
	@Override public boolean touchDown (int x, int y, int pointer, int button) {
		return currentStage.touchDown(x, y, pointer, button);
	}

	@Override public boolean touchUp (int x, int y, int pointer, int button) {
		return currentStage.touchUp(x, y, pointer, button);
	}

	@Override public boolean touchDragged (int x, int y, int pointer) {
		return currentStage.touchDragged(x, y, pointer);
	}

	@Override public boolean keyDown (int keycode) {
		return currentStage.keyDown(keycode);
	}

	@Override public boolean keyUp (int keycode) {
		return currentStage.keyUp(keycode);
	}

	@Override public boolean keyTyped (char character) {
		return currentStage.keyTyped(character);
	}

	@Override public boolean touchMoved (int x, int y) {
		return currentStage.touchMoved(x, y);
	}

	@Override public boolean scrolled (int amount) {
		return currentStage.scrolled(amount);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void pause() {
		isPaused = true;
	}
	
	public void resume() {
		isPaused = false;
	}
	
	public void update(float delta) {
		currentStage.act(delta);
	}
	
	public void draw() {
		currentStage.draw();
	}
}
