package com.jigoku.backend;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.tiled.TileAtlas;
import com.badlogic.gdx.graphics.g2d.tiled.TileMapRenderer;
import com.badlogic.gdx.graphics.g2d.tiled.TiledLoader;
import com.badlogic.gdx.graphics.g2d.tiled.TiledMap;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.jigoku.ApplicationOptions;

public class TileMap {
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	public boolean boundsCorrection;
	
	protected TileMapRenderer renderer;
	protected TiledMap map;
	protected TileAtlas atlas;
	protected OrthographicCamera camera;
	protected Vector3 camDirection;
	protected Vector2 maxCamPosition, minCamPosition;
	protected int tileWidth, tileHeight;
	protected boolean autoscroll;
	
	private Vector3 tmp;
	private boolean isDirty;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	public TileMap(String mapDirectory, String mapName, int tileWidth, int tileHeight) {
		autoscroll = false;
		tmp = new Vector3(0.0f, 0.0f, 0.0f);
		this.tileWidth = tileWidth;
		this.tileHeight = tileHeight;
		map = TiledLoader.createMap(Gdx.files.internal(mapDirectory + mapName + ".tmx"));
		atlas = new TileAtlas(map, Gdx.files.internal(mapDirectory));
		renderer = new TileMapRenderer(map, atlas, tileHeight, tileHeight);
		camera = new OrthographicCamera(ApplicationOptions.APP_WIDTH, ApplicationOptions.APP_HEIGHT);
		
		float minX = ApplicationOptions.APP_WIDTH / 2;
		float minY = ApplicationOptions.APP_HEIGHT / 2;
		float maxX = map.width * map.tileWidth - minX;
		float maxY = map.height * map.tileHeight - minY;
		maxCamPosition = new Vector2(maxX, maxY);
		minCamPosition = new Vector2(minX, minY);
		camDirection = new Vector3(0.0f, 0.0f, 0.0f);
		setPosition(minX, minY);
		boundsCorrection = true;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// MUTATORS / SETTERS
	///////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Sets the amount that the TileMap should scroll automatically.
	 * @param dx is the amount you want the x-value to change over one second.
	 * @param dy is the amount you want the y-value to change over one second.
	 */
	public void setScroll(float dx, float dy) {
		camDirection.set(dx, dy, 0.0f);
		if(dx != 0.0f || dy != 0.0f)
			autoscroll = true;
		else
			autoscroll = false;
	}
	
	/**
	 * Sets the position of the TileMap camera.
	 * @param x is the x-value
	 * @param y is the y-value
	 */
	public void setPosition(float x, float y) {
		camera.position.set(x, y, 0.0f);
		isDirty = true;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void act(float dt) {
		camera.update();
		if(autoscroll) {
			camera.position.add(camDirection.tmp().mul(dt));
			isDirty = true;
		}
		if(isDirty && boundsCorrection) {
			if (camera.position.x < minCamPosition.x) {
				camera.position.x = minCamPosition.x;
			}
			if (camera.position.x > maxCamPosition.x) {
				camera.position.x = maxCamPosition.x;
			}
			if (camera.position.y < minCamPosition.y) {
				camera.position.y = minCamPosition.y;
			}
			if (camera.position.y > maxCamPosition.y) {
				camera.position.y = maxCamPosition.y;
			}
			isDirty = false;
		}
	}
	
	public void draw() {
		renderer.getProjectionMatrix().set(camera.combined);
		tmp.set(0.0f, 0.0f, 0.0f);
		camera.unproject(tmp);
		
		renderer.render(tmp.x, tmp.y, (int)camera.viewportWidth, (int)camera.viewportHeight);
	}
}
