package com.jigoku.debug;

import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.math.MathUtils;
import com.jigoku.backend.Director;
import com.jigoku.objects.PlayerShip;
import com.jigoku.projectiles.BulletHandler;
import com.jigoku.projectiles.InstanceSplitter;
import com.jigoku.projectiles.IntervalSplitter;
import com.jigoku.projectiles.Projectile;
import com.jigoku.projectiles.Splitter;
import com.jigoku.projectiles.Splitter.SplitterPattern;
import com.jigoku.projectiles.Splitter.SplitterType;

public class BulletTestEnemy extends DebugEnemy implements InputProcessor {
	//private SingleSplitter EightWayStarSplitter;
	private PlayerShip player;
	private BulletHandler bulletHandler;
	private Projectile template;
	private Splitter crazySplit;

	public BulletTestEnemy(float x, float y, Director d) {
		super("BulletTestEnemy", x, y, d);
		setRegion(BulletHandler.largeBullet);

		template = new Projectile(BulletHandler.standardBullet, 0.0f, 100.0f, false);
		template.lifespan = 1.0f;
		template.accelleration = -250.0f;
		template.turning = MathUtils.PI;
		crazySplit = new InstanceSplitter(Projectile.StarList(template, 10));
		crazySplit = Splitter.Injection(crazySplit, SplitterPattern.Normal, SplitterType.Destroy, 4);
	}
	
	@Override
	public void create() {
		player = director.getCurrentLevel().player;
		bulletHandler = director.getCurrentLevel().enemyBullets;
	}
	
	@Override
	public boolean keyDown(int keycode) {
		Projectile p;
		///
		///
		/// 
		/*
		 * Note about Splitters: Splitters need only be referenced by a Projectile to use them. The Splitter
		 * itself will clone and create new bullets. calling a new Splitter every Projectile is technically
		 * the improper way of implementing Splitters, and MUST be avoided. I broke that rule here for sake of
		 * testing. When you go to implement splitters, you will create the splitter in the Create() or Constuctors,
		 * and then add them to the Projectile when fired.
		 * 
		 * In a perfect game, no Projectiles OR Splitters will ever need to be created in the act() function.
		 * 
		 * e.g. 
		 * 
		 * Projectile p = bulletHandler.createProjectile(x, y);
		 * if(p != null) {
		 * 		p.setSplitter(SplitterType.Step, myPremadeSplitter);
		 * }
		 * 
		 */
		///
		///
		Splitter s;
		
		switch(keycode) {
		case Keys.NUM_1:
		case Keys.NUM_2:
		case Keys.NUM_3:
		case Keys.NUM_4:
		case Keys.NUM_5:
		case Keys.NUM_6:
		case Keys.NUM_7:
		case Keys.NUM_8:
			p = bulletHandler.createProjectile(this.x, this.y);
			if(p == null)
				return false;
			p.setRegion(BulletHandler.standardBullet);
			template.set(Projectile.Default);
			template.setRegion(BulletHandler.standardBullet);
			template.accelleration = 100.0f;
			break;
		default:
			return false;
		}

		switch(keycode) {
		case Keys.NUM_1:
			//Creates a bullet that moves in a random direction.
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			break;
		case Keys.NUM_2:
			//Creates a bullet that moves towards the player at a set speed.
			p.moveTowardsObject(player, 100.0f);
			break;
		case Keys.NUM_3:
			//Creates a bullet that moves in a random direction.
			//Bullet splits in x at intervals
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			s = new IntervalSplitter(Projectile.SplitByTwo(template), 1.0f, 0.5f, -1);
			p.setSplitter(SplitterType.Step, s);
			break;
		case Keys.NUM_4:
			//Creates a bullet that moves in a random direction.
			//Bullet has a lifespan of x second(s).
			//Bullet splits into y at the end of the bullet's lifespan.
			p.lifespan = 1.0f;
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			s = new InstanceSplitter(Projectile.StarList(template, 8));
			p.setSplitter(SplitterType.Destroy, s);
			break;
		case Keys.NUM_5:
			//Creates a bullet that follows the player with a specific turn radius.
			//Bullet has a lifespan of x second(s).
			//Bullet splits into y at the beginning of bullet's lifespan.
			//Bullet splits into z at the end of bullet's lifespan.
			p.lifespan = 5.0f;
			p.followObject(player, 50.0f);
			s = new InstanceSplitter(Projectile.SplitByTwo(template, MathUtils.PI / 4));
			p.setSplitter(SplitterType.Create, s);
			s = new InstanceSplitter(Projectile.StarList(template, 5));
			p.setSplitter(SplitterType.Destroy, s);
			break;
		case Keys.NUM_6:
			//Creates a bullet that moves in a random direction.
			//Bullet has a lifespan of x second(s).
			p.lifespan = 1.0f;
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			break;
		case Keys.NUM_7:
			//Creates a bullet that moves in a random direction.
			//Bullet has a lifespan of x second(s).
			//Bullet splits into y at the end of bullet's lifespan.
			p.lifespan = 1.0f;
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			template.lifespan = 1.0f;
			template.accelleration = -50.0f;
			s = new InstanceSplitter(Projectile.SplitByTwo(template));
			s = Splitter.Injection(s, SplitterPattern.Normal, SplitterType.Destroy, 2);
			p.setSplitter(SplitterType.Destroy, s);
			break;
		case Keys.NUM_8:
			p.lifespan = 1.0f;
			p.setVelocity(MathUtils.random() * MathUtils.PI * 2, 100.0f);
			p.setSplitter(SplitterType.Destroy, crazySplit);
			break;
		}
		return true;
	}

	@Override
	public boolean keyTyped(char arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean keyUp(int arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean scrolled(int arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchDown(int arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchDragged(int arg0, int arg1, int arg2) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchMoved(int arg0, int arg1) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchUp(int arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		return false;
	}

}
