package com.jigoku.debug;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Rectangle;
import com.jigoku.objects.Object;
import com.jigoku.objects.ObjectHandler;

public class DebugLayer {
	public final static Color DrawColor = new Color(1.0f, 1.0f, 1.0f, 1.0f);
	public final static Color ClearColor = new Color(0.0f, 0.0f, 0.0f, 0.0f);
	public final static Color CoordinateColor = new Color(1.0f, 1.0f, 1.0f, 1.0f);
	public final static Color BoundingBoxColor = new Color(1.0f, 0.0f, 0.0f, 1.0f);
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	public final float height, width;
	private Pixmap pmap;
	private Texture tex;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public DebugLayer(int width, int height) {
		pmap = new Pixmap(width, height, Pixmap.Format.RGBA8888);
		clear();
		tex = new Texture(pmap);
		this.height = height;
		this.width = width;
	}
	
	public DebugLayer() {
		this(MathUtils.nextPowerOfTwo(Gdx.graphics.getWidth()), MathUtils.nextPowerOfTwo(Gdx.graphics.getHeight()));
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void clear() {
		pmap.setColor(DebugLayer.ClearColor);
		pmap.fill();
	}
	
	public void draw(SpriteBatch batch) {
		batch.setColor(DebugLayer.DrawColor);
		tex.draw(pmap, 0, 0);
		batch.draw(tex, 0, 0);
	}
	
	public void getDataFromObjects(ObjectHandler handler) {
		for(Object o : handler.objects)
			getDataFromObject(o);
	}
	
	public void getDataFromObject(Object object) {
		pmap.setColor(DebugLayer.CoordinateColor);
		pmap.drawPixel((int)object.x, (int)(height - object.y));
		Rectangle bb = object.worldBoundingBox();
		pmap.setColor(DebugLayer.BoundingBoxColor);
		pmap.drawRectangle((int)bb.getX(), (int)(height - bb.getY()), (int)bb.getWidth(), (int)-bb.getHeight());
	}
}
