package com.jigoku.objects;

import com.jigoku.backend.AbstractObject;
import com.jigoku.backend.AnimationExtended;
import com.jigoku.backend.Director;
import com.jigoku.projectiles.BulletHandler;

public class Object extends AbstractObject {
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	public boolean flipX;
	public boolean flipY;
	public float offsetX;
	public float offsetY;
	protected AnimationExtended currentAnimation;
	protected float animationTime;
	protected boolean isAnimated;
	protected boolean animationIsReversed;
	protected boolean currentAnimationLooping;
	protected Director director;
	public ObjectHandler handler;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS
	///////////////////////////////////////////////////////////////////////////////////////////////
	@Override
	public Object clone() {
		return new Object(this);
	}
	
	public Object(Object o) {
		super(o);
		
		//Object
		this.flipX = o.flipX;
		this.flipY = o.flipY;
		this.offsetX = o.offsetX;
		this.offsetY = o.offsetY;
		this.currentAnimation = o.currentAnimation;
		this.animationTime = o.animationTime;
		this.isAnimated = o.isAnimated;
		this.animationIsReversed = o.animationIsReversed;
		this.currentAnimationLooping = o.currentAnimationLooping;
		this.director = o.director;
		
		//Handled by ObjectHandler
		this.handler = null;
	}
	
	public Object(String name, Director d) {
		super(name);

		//Object
		this.flipX = false;
		this.flipY = false;
		this.offsetX = 0.0f;
		this.offsetY = 0.0f;
		this.currentAnimation = null;
		this.animationTime = 0.0f;
		this.isAnimated = false;
		this.animationIsReversed = false;
		this.currentAnimationLooping = false;
		this.director = d;
		
		//Handled by ObjectHandler
		this.handler = null;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// MUTATORS / SETTERS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void setAnimation(AnimationExtended animation, boolean looping, boolean reversed) {
		if(currentAnimation == animation)
			return;
		animationTime = (reversed) ? animation.totalPlaytime : 0.0f;
		isAnimated = true;
		currentAnimation = animation;
		currentAnimationLooping = looping;
		animationIsReversed = reversed;
		setRegion(currentAnimation.getKeyFrame(0.0f, false));
	}

	
	public void setOffset(float ox, float oy) {
		this.offsetX = ox;
		this.offsetY = oy;
	}

	public void setAnimationIsReversed(boolean r) {
		animationIsReversed = r;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// ACCESSORS / GETTERS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public Director getDirector() {
		return director;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// OVERRIDE
	///////////////////////////////////////////////////////////////////////////////////////////////
	@Override
	public void create() {}

	@Override
	public void destroy() {
		handler.removeObject(this);
	}
	
	@Override
	public void act(float dt) {
		stateTime += dt;

		if(isAnimated) {
			if(!animationIsReversed) {
				animationTime += dt;
				if(animationTime >= currentAnimation.totalPlaytime) {
					animationTime -= currentAnimation.totalPlaytime;
					endOfAnimation();
				}
			} else {
				animationTime -= dt;
				if(animationTime < 0.0f) {
					animationTime += currentAnimation.totalPlaytime;
					endOfAnimation();
				}
			}
			this.setRegion(currentAnimation.getKeyFrame(animationTime, currentAnimationLooping));
		}
	}

	@Override
	public void endOfAnimation() {}

	@Override
	public void checkCollisionWithObject(AbstractObject other) {
		if(Object.collisionTest(this, other)) {
			this.collisionHasOccurredWithObject(other);
			other.collisionHasOccurredWithObject(this);
		}
	}

	@Override
	public void checkCollisionWithObjects(ObjectHandler handler) {
		for(Object o : handler.objects)
			this.checkCollisionWithObject(o);
	}
	
	public void checkCollisionWithProjectiles(BulletHandler handler) {
		handler.checkCollisionWithObject(this);
	}
	
	@Override
	public void collisionHasOccurredWithObject(AbstractObject other) {}
}