package com.jigoku.objects;

import com.jigoku.ApplicationOptions;
import com.jigoku.backend.AnimationExtended;
import com.jigoku.backend.Director;
import com.jigoku.backend.Helper;
import com.jigoku.projectiles.BulletHandler;
import com.jigoku.projectiles.Projectile;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;

public class PlayerShip extends Object implements InputProcessor {
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// STATIC LOAD VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	protected static TextureAtlas atlas = null;
	//TODO Set back to protected
	public static AnimationExtended aFlight, aFlightToFight, aFightToFlight, aFight,
										aFlightToRight, aFlightToLeft, aFlightRight, aFlightLeft;
	final static float flyingSpeed = 240.0f;
	final static float fightingSpeed = 120.0f;
	final static float bulletSpeed = 640.0f;
	final static float shootingCooldownDefault = 0.1f;
	final static int maxBulletStreams = 1;

	///////////////////////////////////////////////////////////////////////////////////////////////
	// SPECIFIC OBJECT ENUMS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public enum FlightMode {
		init,
		Flying,
		Fighting,
	}
	
	public enum LogicState {
		init,
		FlyingStraight,
		FlyingToLeft,
		FlyingFromLeft,
		FlyingLeft,
		FlyingToRight,
		FlyingFromRight,
		FlyingRight,
		FlyingToFightingStraight,
		FlyingToFightingLeft,
		FlyingToFightingRight,
		FightingToFlyingStraight,
		FightingToFlyingLeft,
		FightingToFlyingRight,
		FightingStraight,
		FightingToLeft,
		FightingFromLeft,
		FightingLeft,
		FightingToRight,
		FightingFromRight,
		FightingRight,
	}

	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	private float speed;
	private float bulletCooldown;
	private boolean kDown, kLeft, kRight, kUp, kShoot;
	private FlightMode currentFlightMode;
	private LogicState currentLogicState;
	private BulletHandler playerBullets;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS / STATIC LOAD AND UNLOAD
	///////////////////////////////////////////////////////////////////////////////////////////////
	public PlayerShip(float x, float y, BulletHandler playerBulletHandler, Director d) {
		//Prepare object for initialization
		super("Player", d);
		PlayerShip.load();
		currentFlightMode = FlightMode.init;
		currentLogicState = LogicState.init;
		kDown = kLeft = kRight = kUp = kShoot = false;
		
		//Begin initializing object
		setPosition(x, y);
		setFlightMode(FlightMode.Flying);
		setLogicState(LogicState.FlyingStraight);
		setOrigin(width / 2, height / 2);
		
		this.playerBullets = playerBulletHandler;
		this.bulletCooldown = 0.0f;
	}
	
	public static void load() {
		if(atlas == null) {
			atlas = new TextureAtlas("data/sprites/playerShip.txt");
			aFlight = Helper.animationFromAtlas(atlas, "playerShipFlightMode", 0.05f);
			aFlightToFight = Helper.animationFromAtlas(atlas, "playerShipFlightToFight", 0.1f);
			aFight = Helper.animationFromAtlas(atlas, "playerShipFightMode", 0.05f);
			aFightToFlight = Helper.animationFromAtlas(atlas, "playerShipFightToFlight", 0.1f);
			aFlightToLeft = Helper.animationFromAtlas(atlas, "playerShipFlightToLeft", 0.1f);
			aFlightToRight = Helper.animationFromAtlas(atlas, "playerShipFlightToRight", 0.1f);
			aFlightLeft = Helper.animationFromAtlas(atlas, "playerShipFlightLeft", 0.1f);
			aFlightRight = Helper.animationFromAtlas(atlas, "playerShipFlightRight", 0.1f);
		}
	}
	
	public static void unload() {
		
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// ENUM STATE HANDLING
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	public void setFlightMode(FlightMode m) {
		if(currentFlightMode == m)
			return;
		switch(m) {
		case Flying:
			speed = PlayerShip.flyingSpeed;
			switch(currentLogicState) {
			case FlyingToLeft:
			case FlyingLeft:
				setLogicState(LogicState.FlyingToFightingLeft);
				break;
			case FlyingToRight:
			case FlyingRight:
				setLogicState(LogicState.FlyingToFightingRight);
				break;
			default:
				setLogicState(LogicState.FlyingToFightingStraight);
				break;
			}
			break;
		case Fighting:
			speed = PlayerShip.fightingSpeed;
			switch(currentLogicState) {
			case FightingToLeft:
			case FightingLeft:
				setLogicState(LogicState.FightingToFlyingLeft);
				break;
			case FightingToRight:
			case FightingRight:
				setLogicState(LogicState.FightingToFlyingRight);
				break;
			default:
				setLogicState(LogicState.FightingToFlyingStraight);
				break;
			}
			break;
		}
		currentFlightMode = m;
	}
	
	public void setLogicState(LogicState s) {
		if(currentLogicState == s)
			return;
		switch(s) {
		case FlyingStraight:
			setAnimation(aFlight, true, false);
			break;
		case FlyingToLeft:
			setAnimation(aFlightToLeft, false, false);
			break;
		case FlyingFromLeft:
			if(currentLogicState == LogicState.FlyingToLeft)
				setAnimationIsReversed(true);
			else
				setAnimation(aFlightToLeft, false, true);
			break;
		case FlyingLeft:
			setAnimation(aFlightLeft, true, false);
			break;
		case FlyingToRight:
			setAnimation(aFlightToRight, false, false);
			break;
		case FlyingFromRight:
			if(currentLogicState == LogicState.FlyingToRight)
				setAnimationIsReversed(true);
			else
				setAnimation(aFlightToRight, false, true);
			break;
		case FlyingRight:
			setAnimation(aFlightRight, true, false);
			break;
		case FlyingToFightingStraight:
			setAnimation(aFlightToFight, false, false);
			break;
		case FlyingToFightingLeft:
			break;
		case FlyingToFightingRight:
			break;
		case FightingToFlyingStraight:
			break;
		case FightingToFlyingLeft:
			break;
		case FightingToFlyingRight:
			break;
		case FightingStraight:
			setAnimation(aFight, true, false);
			break;
		case FightingToLeft:
			break;
		case FightingFromLeft:
			break;
		case FightingLeft:
			break;
		case FightingToRight:
			break;
		case FightingFromRight:
			break;
		case FightingRight:
			break;
		}
		currentLogicState = s;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	
	
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// OVERRIDES
	///////////////////////////////////////////////////////////////////////////////////////////////
	@Override
	public void endOfAnimation() {
		switch(currentLogicState) {
		case FlyingToLeft:
			setLogicState(LogicState.FlyingLeft);
			break;
		case FlyingFromRight:
		case FlyingFromLeft:
			setLogicState(LogicState.FlyingStraight);
			break;
		case FlyingToRight:
			setLogicState(LogicState.FlyingRight);
			break;
		case FlyingToFightingStraight:
			setLogicState(LogicState.FightingStraight);
			break;
		case FlyingToFightingLeft:
			setLogicState(LogicState.FightingLeft);
			break;
		case FlyingToFightingRight:
			setLogicState(LogicState.FightingRight);
			break;
		case FightingToFlyingStraight:
			setLogicState(LogicState.FlyingStraight);
			break;
		case FightingToFlyingLeft:
			setLogicState(LogicState.FlyingLeft);
			break;
		case FightingToFlyingRight:
			setLogicState(LogicState.FlyingRight);
			break;
		case FightingToLeft:
			setLogicState(LogicState.FightingLeft);
			break;
		case FightingFromLeft:
		case FightingFromRight:
			setLogicState(LogicState.FightingStraight);
			break;
		case FightingToRight:
			setLogicState(LogicState.FightingRight);
			break;
		default:
			break;
		}
	}
	
	@Override
	public void act(float dt) {
		super.act(dt);
		
		if(bulletCooldown > 0.0f)
			bulletCooldown -= dt;
		
		//Handle user input
		float translate[] = {0.0f, 0.0f};
		if(kLeft)
			translate[0] -= speed;
		if(kRight)
			translate[0] += speed;
		if(kUp)
			translate[1] += speed;
		if(kDown)
			translate[1] -= speed;
		if(kShoot && bulletCooldown <= 0.0f) {
			//TODO: Add bullets
			Projectile p = playerBullets.createProjectile(this.x, this.y);
			if(p != null) {
				p.setRegion(BulletHandler.standardBullet);
				p.setMotionY(PlayerShip.bulletSpeed);
				bulletCooldown = PlayerShip.shootingCooldownDefault;
			}
		}

		//Handle "Rocking" animation
		if(translate[0] > 0.0f) {
			switch(currentLogicState) {
			case FlyingStraight:
				setLogicState(LogicState.FlyingToRight);
				break;
			case FlyingToLeft:
			case FlyingLeft:
				setLogicState(LogicState.FlyingFromLeft);
				break;
			}
		} else if (translate[0] < 0.0f) {
			switch(currentLogicState) {
			case FlyingStraight:
				setLogicState(LogicState.FlyingToLeft);
				break;
			case FlyingToRight:
			case FlyingRight:
				setLogicState(LogicState.FlyingFromRight);
				break;
			}
		} else {
			switch(currentLogicState) {
			case FlyingRight:
			case FlyingToRight:
				setLogicState(LogicState.FlyingFromRight);
				break;
			case FlyingLeft:
			case FlyingToLeft:
				setLogicState(LogicState.FlyingFromLeft);
				break;
			}
		}
		
		//Apply actual translations
		this.translate(translate[0] * dt, translate[1] * dt);
		float halfwidth = width / 2;
		if(this.x < halfwidth)
			this.x = halfwidth;
		if(this.x > ApplicationOptions.APP_WIDTH - halfwidth)
			this.x = ApplicationOptions.APP_WIDTH - halfwidth;
		if(this.y < 0.0f)
			this.y = 0.0f;
		if(this.y > ApplicationOptions.APP_HEIGHT - this.height)
			this.y = ApplicationOptions.APP_HEIGHT - this.height;
	}

	@Override
	public boolean keyDown(int keycode) {
		if(keycode == Keys.LEFT)
			kLeft = true;
		if(keycode == Keys.RIGHT)
			kRight = true;
		if(keycode == Keys.UP)
			kUp = true;
		if(keycode == Keys.DOWN)
			kDown = true;
		if(keycode == Keys.Z)
			kShoot = true;
		//if(keycode == Keys.SPACE)
			//TODO Add switch ability back in.
			//Perhaps question removing the switch alltogether?
		return true;
	}

	@Override
	public boolean keyUp(int keycode) {
		if(keycode == Keys.LEFT)
			kLeft = false;
		if(keycode == Keys.RIGHT)
			kRight = false;
		if(keycode == Keys.UP)
			kUp = false;
		if(keycode == Keys.DOWN)
			kDown = false;
		if(keycode == Keys.Z) {
			kShoot = false;
		}
		return true;
	}

	@Override
	public boolean keyTyped(char arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean scrolled(int arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchDown(int arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchDragged(int arg0, int arg1, int arg2) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchMoved(int arg0, int arg1) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchUp(int arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		return false;
	}
}
