package com.jigoku.projectiles;

public abstract class Splitter {
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// SPECIFIC SPLITTER ENUMS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public static enum SplitterType {
		Create,
		Step,
		Destroy,
	}
	
	public static enum SplitterPattern {
		Normal,
		Firework,
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// INSTANCE VARIABLES
	///////////////////////////////////////////////////////////////////////////////////////////////
	public Projectile[] splitList;
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// CONSTRUCTORS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public abstract Splitter clone();
	
	public Splitter(Splitter s) {
		splitList = new Projectile[s.splitList.length];
		for(int i = 0; i < s.splitList.length; i++)
			splitList[i] = s.splitList[i].clone();
	}
	
	public Splitter(Projectile[] splitList) {
		this.splitList = splitList;
	}

	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public void actuate(Projectile parent) {
		for(Projectile p : splitList) {
			Projectile t = parent.handler.createProjectile(p);
			if(t != null)
				parent.influenceProjectile(t);
			else
				break;
		}
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////
	// PUBLIC STATIC METHODS
	///////////////////////////////////////////////////////////////////////////////////////////////
	public static Splitter duplicate(Splitter s) {
		if(s == null)
			return null;
		return s.clone();
	}
	
	public static Splitter Injection(Splitter template, SplitterPattern pattern, SplitterType type, int levels) {
		return InjectionRecursive(template.clone(), type, pattern, levels);
	}
	
	private static Splitter InjectionRecursive(Splitter top, SplitterType type, SplitterPattern pattern, int level) {
		if(level <= 1)
			return top;
		Splitter current = InjectionRecursive(top.clone(), type, pattern, level - 1);
		
		//Apply pattern to current layer utilizing template
		switch(pattern) {
		case Normal:
			for(Projectile p : top.splitList)
				p.setSplitter(type, current);
			break;
		}
		
		return top;
	}
}
